
<!DOCTYPE html>
<html ng-app="Static">
<head>
    <meta charset="UTF-8" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge" />
    <title>Learning Assemble, Step 3 - Grunt Workflow | James Wing</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta name="description" content="Part of a series exploring the Assemble static web site framework for Node.js" />

    

<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/font-awesome/4.1.0/css/font-awesome.min.css"/>

<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.5/css/bootstrap.min.css" />



<link rel="stylesheet" type="text/css" href="../assets/css/compiled/site.css">



<!--<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/highlight.js/8.6/styles/default.min.css">-->


<link href="../assets/prism/prism.css" rel="stylesheet" />

<!--[if lt IE 9]>
<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->

<script src="//ajax.googleapis.com/ajax/libs/jquery/1.10.2/jquery.min.js"></script>


</head>
<body id="blogpost" ng-controller="BaseController">



<header class="navbar navbar-inverse normal" role="banner">
    <div class="navbar-container">
        <div class="navbar-header">
            <button class="navbar-toggle" type="button" data-toggle="collapse" data-target=".bs-navbar-collapse">
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            <a href="/index.html" class="navbar-brand">James Wing</a>
        </div>
        <nav class="collapse navbar-collapse bs-navbar-collapse" role="navigation">
            <ul class="nav navbar-nav navbar-right">
                

<li><a href="/index.html">Blog</a></li>
<li><a href="/about.html">About</a></li>


            </ul>
        </nav>
    </div>
</header>



<div class="layout-sidebar">
    <div class="row">
        <div class="col-sm-12 col-md-9 col-lg-8">
            

<div class="blogpost-wrapper">
    

    <div class="row">
        <div class="col-md-10 col-lg-10 post">
            <h1 class="title">
                <a href="">Learning Assemble, Step 3 - Grunt Workflow</a>
            </h1>
            <!--<div class="author">-->
                <!---->
                <!---->
                <!---->
                <!--2014-10-24 by James Wing-->
            <!--</div>-->
            <div class="date">
                2014-10-24
            </div>

            

            <div class="content">

                
                    <p>Our static web site project needs a lot more than just Assemble, and we&#x27;re going to take a quick break to set up
some non-Assemble tasks in Grunt.
We need Grunt to set up a useful workflow, including some way of previewing and debugging our static site.
In this post, we&#x27;ll go through the configuration of some additional Grunt tasks for a basic static site workflow.</p>
<ul>
<li>Watch</li>
<li>Connect</li>
<li>Combined Build and Server Tasks</li>
<li>Livereload</li>
<li>Clean</li>
</ul>

                    <h2 id="watch">Watch</h2>
<p>Watch (<a href="https://github.com/gruntjs/grunt-contrib-watch">grunt-contrib-watch</a>) helps update your output files as you
make changes to the source files.  Watch is the foundation for a very convenient preview and feedback cycle.
Add watch to your project using NPM:</p>
<pre>
npm install grunt-contrib-watch --save-dev
</pre>

<p>Then update Gruntfile.js to configure Watch:</p>
<pre class="language-javascript line-numbers" data-line="15,16,17,18,19,20,23" data-start="1"><code>
module.exports = function (grunt) {
    grunt.initConfig({
        assemble: {
            hello: {
                options: {
                    layoutdir: 'source/templates/layouts',
                    layout: 'default.hbs',
                    partials: 'source/templates/partials/**/*.hbs'
                },
                files: [
                    { expand: true, cwd: 'source/templates/pages', src: '**/*.hbs', dest: 'output/' }
                ]
            }
        },
        watch: {
            templates: {
                files: ['source/templates/**/*.hbs'],
                tasks: ['assemble']
            }
        }
    });
    grunt.loadNpmTasks('assemble');
    grunt.loadNpmTasks('grunt-contrib-watch');
};
</code></pre>

<p>At the bottom of the file, we loaded the default task for grunt-contrib-watch, which is executed as
<code>grunt watch</code>.  This starts the never-ending watch loop, based on the configuration you supply about which files to
watch and what to do when they change.  We specified a file spec for our Handlebars templates, and to run the &#39;assemble&#39;
task when any of those change.  As our project grows, we will have many more targets under the watch configuration,
each specifying some group of source files and their related processing commands.</p>
<p>Go ahead and try running <code>grunt watch</code>.</p>
<pre>
$ grunt watch
Running "watch" task
Waiting...
</pre>

<p>It doesn&#39;t look like much.  You have to make a change to a file to see some action.  Let&#39;s type some gibberish in the index.hbs page template, and save the file.</p>
<pre>
&gt;&gt; File "source/templates/pages/index.hbs" changed.
Running "assemble:hello" (assemble) task
Assembling output/about.html OK
Assembling output/features.html OK
Assembling output/index.html OK
Assembling output/pricing.html OK
&gt;&gt; 4 pages assembled.

Done, without errors.
Completed in 1.999s at Thu Oct 23 2014 14:35:51 GMT-0700 (PDT) - Waiting...
</pre>

<p>Not bad at all.  This goes on forever until you stop it with Ctrl-C, or the process crashes.</p>
<h2 id="connect">Connect</h2>
<p>Connect (<a href="https://github.com/gruntjs/grunt-contrib-connect">grunt-contrib-connect</a>) is a very useful web server
component for Grunt.  It sets up a simple server so we don&#39;t have to have any web server code of our own.  Since we
are only deploying the static content, we don&#39;t need anything more complicated.  Add Connect to your project using NPM:</p>
<pre>
npm install grunt-contrib-connect --save-dev
</pre>

<p>Then update Gruntfile.js to configure Connect:</p>
<pre class="language-javascript line-numbers" data-line="15,16,17,18,19,20,21,22,23,24,33" data-start="1"><code>
module.exports = function (grunt) {
    grunt.initConfig({
        assemble: {
            hello: {
                options: {
                    layoutdir: 'source/templates/layouts',
                    layout: 'default.hbs',
                    partials: 'source/templates/partials/**/*.hbs'
                },
                files: [
                    { expand: true, cwd: 'source/templates/pages', src: '**/*.hbs', dest: 'output/' }
                ]
            }
        },
        connect: {
            preview: {
                options: {
                    base: ['output'],
                    port: 9000,
                    hostname: 'localhost',
                    keepalive: true
                }
            }
        },
        watch: {
            templates: {
                files: ['source/templates/**/*.hbs'],
                tasks: ['assemble']
            }
        }
    });
    grunt.loadNpmTasks('assemble');
    grunt.loadNpmTasks('grunt-contrib-connect');
    grunt.loadNpmTasks('grunt-contrib-watch');
};
</code></pre>

<p>Our Connect task configuration specifies that Connect will serve files from the &#39;output&#39; folder to <a href="http://localhost:9000/">http://localhost:9000/</a>.
Try running <code>grunt connect</code> and aiming a browser at localhost:9000 to see our generated pages.
Because we specified the &#39;keepalive: true&#39; option, Connect keeps running forever, until we shut it down,
just like Watch did.  If we were using Connect in isolation, that would be great.
Instead, we want Watch and Connect to play nice together.</p>
<h2 id="combined-build-and-server-tasks">Combined Build and Server Tasks</h2>
<p>Grunt supports alias tasks that will run run multiple Grunt tasks in sequence.  We&#39;re going to use these to coordinate
our individual tasks into workflows.  Let&#39;s create two to start, one to build our output files, and one to run the
preview server.</p>
<pre class="language-javascript line-numbers" data-line="21,35,36" data-start="1"><code>
module.exports = function (grunt) {
    grunt.initConfig({
        assemble: {
            hello: {
                options: {
                    layoutdir: 'source/templates/layouts',
                    layout: 'default.hbs',
                    partials: 'source/templates/partials/**/*.hbs'
                },
                files: [
                    { expand: true, cwd: 'source/templates/pages', src: '**/*.hbs', dest: 'output/' }
                ]
            }
        },
        connect: {
            preview: {
                options: {
                    base: ['output'],
                    port: 9000,
                    hostname: 'localhost',
                    keepalive: false
                }
            }
        },
        watch: {
            templates: {
                files: ['source/templates/**/*.hbs'],
                tasks: ['assemble']
            }
        }
    });
    grunt.loadNpmTasks('assemble');
    grunt.loadNpmTasks('grunt-contrib-connect');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.registerTask('build', ['assemble']);
    grunt.registerTask('server', ['build','connect','watch']);
};
</code></pre>

<p>You can see our alias tasks at the bottom of the file, created with the registerTask function.</p>
<ul>
<li>Our <strong>build</strong> task is a bit light for now, it just runs Assemble.  But it will grow.</li>
<li>The <strong>server</strong> task first runs the build to baseline the output, then starts a Connect server, and finally
runs Watch to keep everything going in a monitoring loop.  We are now changing the Connect &#39;keepalive&#39; setting to
false so that Connect will defer to Watch.</li>
</ul>
<p>Give this a try by running <code>grunt server</code>.  Point your browser at localhost:9000.  Modify some of the page files to see
Watch run Assemble after changes, and refresh your browser to see the results.  You&#39;ll notice how painful it was to
push the refresh button on your browser, and you&#39;ll be happy to know that the Livereload feature can help relieve you
of that heavy burden.</p>
<h2 id="livereload">Livereload</h2>
<p>Livereload is an optional feature of grunt-contrib-connect and grunt-contrib-watch that maintains a socket connection with your browser to cause it to refresh when server files have changed.  If you can put your browser on a second screen, you can watch your static site auto-magically update as you work.  All we need is some additional Gruntfile configuration:</p>
<pre class="language-javascript line-numbers" data-line="22,31,32,33,34,35,36" data-start="1"><code>
module.exports = function (grunt) {
    grunt.initConfig({
        assemble: {
            hello: {
                options: {
                    layoutdir: 'source/templates/layouts',
                    layout: 'default.hbs',
                    partials: 'source/templates/partials/**/*.hbs'
                },
                files: [
                    { expand: true, cwd: 'source/templates/pages', src: '**/*.hbs', dest: 'output/' }
                ]
            }
        },
        connect: {
            preview: {
                options: {
                    base: ['output'],
                    port: 9000,
                    hostname: 'localhost',
                    keepalive: false,
                    livereload: 35729
                }
            }
        },
        watch: {
            templates: {
                files: ['source/templates/**/*.hbs'],
                tasks: ['assemble']
            },
            livereload: {
                options: {
                    livereload: '<%= connect.preview.options.livereload %>'
                },
                files: ['output/**.*']
            }
        }
    });
    grunt.loadNpmTasks('assemble');
    grunt.loadNpmTasks('grunt-contrib-connect');
    grunt.loadNpmTasks('grunt-contrib-watch');

    grunt.registerTask('build', ['assemble']);
    grunt.registerTask('server', ['build','connect','watch']);
};
</code></pre>

<p>We made only a few changes.  We added the livereload port number to the Connect configuration.  We created a new
Watch target, to watch the output files, and set it up to reference the livereload port from the Connect configuration.
Specifying a port effectively turns it on.</p>
<p>Try repeating your exercise of running the <code>grunt server</code> task, opening a page in your browser, then saving a change
to the source page file.  Not only will Watch run Assemble to compile your page, but livereload will refresh your browser.
If you are curious, view the source of the page from your browser, you can see the script injected for livereload.</p>
<pre><code class="language-markup">&lt;!DOCTYPE html&gt;
&lt;html&gt;
&lt;head&gt;
    &lt;title&gt;Home&lt;/title&gt;
&lt;/head&gt;
&lt;body&gt;
&lt;div&gt;
    &lt;a href=&quot;/&quot;&gt;Home&lt;/a&gt; |
    &lt;a href=&quot;/features.html&quot;&gt;Features&lt;/a&gt; |
    &lt;a href=&quot;/pricing.html&quot;&gt;Pricing&lt;/a&gt; |
    &lt;a href=&quot;/about.html&quot;&gt;About&lt;/a&gt;
&lt;/div&gt;

&lt;h1&gt;Home&lt;/h1&gt;
&lt;p&gt;This is the home page&lt;/p&gt;


&lt;script&gt;//&lt;![CDATA[
document.write(&#39;&lt;script src=&quot;//&#39; + (location.hostname || &#39;localhost&#39;) + &#39;:35729/livereload.js?snipver=1&quot;&gt;&lt;\/script&gt;&#39;)
//]]&gt;&lt;/script&gt;
&lt;/body&gt;
&lt;/html&gt;</code></pre>
<h2 id="clean">Clean</h2>
<p>Last, but not least, <a href="https://github.com/gruntjs/grunt-contrib-clean">grunt-contrib-clean</a> will help us make sure our
output is the true reflection of generating the recent source files.  Add grunt-contrib-clean to the project:</p>
<pre>
npm install grunt-contrib-clean --save-dev
</pre>

<p>For now, we&#39;ll just run Clean at the start of the <strong>build</strong> task.  Configuring Clean is pretty simple, it
accepts a list of file specifications to be cleaned out, which is just the output folder and its contents for now.
In a complex workflow, we might want to be careful about deleting everything from the output because of the time
required to regenerate it all.  We don&#39;t have that problem yet.</p>
<pre class="language-javascript line-numbers" data-line="15,41,45" data-start="1"><code>
module.exports = function (grunt) {
    grunt.initConfig({
        assemble: {
            hello: {
                options: {
                    layoutdir: 'source/templates/layouts',
                    layout: 'default.hbs',
                    partials: 'source/templates/partials/**/*.hbs'
                },
                files: [
                    { expand: true, cwd: 'source/templates/pages', src: '**/*.hbs', dest: 'output/' }
                ]
            }
        },
        clean: ['output/**'],
        connect: {
            preview: {
                options: {
                    base: ['output'],
                    port: 9000,
                    hostname: 'localhost',
                    keepalive: false,
                    livereload: 35729
                }
            }
        },
        watch: {
            templates: {
                files: ['source/templates/**/*.hbs'],
                tasks: ['assemble']
            },
            livereload: {
                options: {
                    livereload: '<%= connect.preview.options.livereload %>'
                },
                files: ['output/**.*']
            }
        }
    });
    grunt.loadNpmTasks('assemble');
    grunt.loadNpmTasks('grunt-contrib-clean');
    grunt.loadNpmTasks('grunt-contrib-connect');
    grunt.loadNpmTasks('grunt-contrib-watch');

    grunt.registerTask('build', ['clean', 'assemble']);
    grunt.registerTask('server', ['build','connect','watch']);
};
</code></pre>

<p>Give that a try.  In the next post, we&#39;ll get back to Assemble.</p>
<p>For the code, please see the <a href="https://github.com/jvwing/learning-assemble">learning-assemble</a> repository on GitHub.
The branch <a href="https://github.com/jvwing/learning-assemble/tree/step03-grunt-workflow">step03-grunt-workflow</a>
contains the completed code from this post.</p>

                

                
                    
                
                    
                
                    
                
                    
                        <div class="blog-post-next">
                            Next:
                            <a href="/learning-assemble/step-4-data.html">
                                Learning Assemble, Step 4 - Data <i class="fa fa-lg fa-arrow-right"></i>
                            </a>
                        </div>
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                

            </div>

            <div class="share">
                <a href="https://twitter.com/share" class="twitter-share-button" data-text="Learning Assemble, Step 3 - Grunt Workflow" data-via="jvwing" data-size="large">Tweet</a>
                <script>!function(d,s,id){var js,fjs=d.getElementsByTagName(s)[0],p=/^http:/.test(d.location)?'http':'https';if(!d.getElementById(id)){js=d.createElement(s);js.id=id;js.src=p+'://platform.twitter.com/widgets.js';fjs.parentNode.insertBefore(js,fjs);}}(document, 'script', 'twitter-wjs');</script>
            </div>

            <hr/>
            <div class="other-posts">
                <h4>You might also enjoy:</h4>
                <ul>
                    
                        <li><a href="/learning-assemble/step-1-hello-assemble.html">Learning Assemble, Step 1 - Hello, Assemble!</a></li>
                    
                        <li><a href="/learning-assemble/step-2-basic-templating.html">Learning Assemble, Step 2 - Basic Templating</a></li>
                    
                        <li><a href="/learning-assemble/step-3-grunt-workflow.html">Learning Assemble, Step 3 - Grunt Workflow</a></li>
                    
                        <li><a href="/learning-assemble/step-4-data.html">Learning Assemble, Step 4 - Data</a></li>
                    
                        <li><a href="/learning-assemble/step-5-page-navigation.html">Learning Assemble, Step 5 - Page Navigation</a></li>
                    
                        <li><a href="/learning-assemble/step-6-blogging-with-markdown.html">Learning Assemble, Step 6 - Blogging with Markdown</a></li>
                    
                        <li><a href="/learning-assemble/step-7-custom-handlebars-helpers.html">Learning Assemble, Step 7 - Custom Handlebars Helpers</a></li>
                    
                        <li><a href="/learning-assemble/step-8-debugging-assemble.html">Learning Assemble, Step 8 - Debugging Assemble</a></li>
                    
                        <li><a href="/learning-assemble/step-9-sitemap.html">Learning Assemble, Step 9 - Sitemap</a></li>
                    
                        <li><a href="/learning-assemble/step-10-pages-from-data.html">Learning Assemble, Step 10 - Pages from Data</a></li>
                    
                        <li><a href="/learning-assemble/step-11-more-navigation.html">Learning Assemble, Step 11 - More Navigation</a></li>
                    
                        <li><a href="/learning-assemble/step-12-navigation-helper.html">Learning Assemble, Step 12 - Navigation Helper</a></li>
                    
                </ul>
            </div>

            <div class="comments">
                <div id="disqus_thread"></div>
<script type="text/javascript">
    /* * * CONFIGURATION VARIABLES * * */
    var disqus_shortname = 'jameswingnet';

    /* * * DON'T EDIT BELOW THIS LINE * * */
    (function() {
        var dsq = document.createElement('script'); dsq.type = 'text/javascript'; dsq.async = true;
        dsq.src = '//' + disqus_shortname + '.disqus.com/embed.js';
        (document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(dsq);
    })();
</script>
<noscript>Please enable JavaScript to view the <a href="https://disqus.com/?ref_noscript" rel="nofollow">comments powered by Disqus.</a></noscript>
            </div>

        </div>

    </div>

</div>

        </div>
        <div class="col-md-3 col-lg-4 sidebar">
            <div class="posts-list-brief">
    <h3 class="sidebar-heading">Recent</h3>
    <ul>
        
            
        
            
                
                    <li>
                        
                            <a href="/aws/query-cloudtrail-events-with-athena.html">Query CloudTrail Events with Athena</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/nifi/nifi-internal-fields.html">Hidden Fields in Apache NiFi FlowFiles</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/nifi/first-look-at-apache-nifi.html">First Look at Apache NiFi</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/monitoring/monitoring-advice-distilled.html">Monitoring Advice Distilled</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/aws/using-codecommit-and-git-credentials.html">Using CodeCommit and GitHub Credential Helpers</a>
                        
                    </li>
                
            
        
            
        
            
        
            
        
            
        
    </ul>
</div>
            <hr/>
            <div class="ads">
    <!--<h3 class="sidebar-heading">Featured</h3>-->
    <div class="ad">
        <a href="http://prontolyzer.jameswing.net/">
            <img class="thumbnail" src="/assets/ads/prontolyzer-thumb.png" />
            <h4 class="title">Prontolyzer</h4>
            <p class="description">
                Interactive mapping and exploration of bike trip data for the 2015 Pronto Cycle Share Data Challenge.
                
            </p>
        </a>
    </div>
    <hr/>
    <div class="ad">
        <a href="/troubleshooting-chart/introducing-the-troubleshooting-chart.html">
            <img class="thumbnail" src="/assets/posts/troubleshooting-chart/troubleshootingchartannotated-thumbnail.png" />
            <h4 class="title">The Troubleshooting Chart</h4>
            <p class="description">
                Level up your troubleshooting with simple data analysis and visualizations.
                A thrill-ride for Ops types.
            </p>
        </a>
    </div>
</div>
        </div>
    </div>
</div>




<div id="footer">
    <div class="container">
        <div class="row">
            <div class="col-sm-6 copyright">
                © 2014-2017 James Wing. All Rights Reserved.
            </div>
            <div class="col-sm-6 menu">
                <ul>
                    

<li><a href="/index.html">Blog</a></li>
<li><a href="/about.html">About</a></li>


                </ul>
            </div>
        </div>
    </div>
</div>






<script src="//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.5/js/bootstrap.min.js"></script>

<script>
    (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
        (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
            m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
    })(window,document,'script','//www.google-analytics.com/analytics.js','ga');
    var gaId = 'UA-6573';
    gaId += '6738-1';
    ga('create', gaId, 'auto');
    ga('send', 'pageview');

</script>









<script src="../assets/prism/prism.js"></script>


</body>
</html>